<?php

if( !defined( '_VALID_MOS' ) && !defined( '_JEXEC' ) ) die( 'Direct Access to '.basename(__FILE__).' is not allowed.' );

class mod_virtuemart_acumulus
{
	public $p_tax = 21;
	/**
	 * Constructor
	 *
	 * @param object $d
	 * @return void
	 */
	public function __construct($d, $key)
	{	
		if(is_array($key)){
			$key = $key[0];
		}
	
		if(!in_array('curl', get_loaded_extensions()))
		{
			JError::raiseWarning('WARNING', JText::_('Voor het gebruik van de Acumulus extensie dient CURL actief te zijn op uw server.'));
		}
		
		$response = $this->_postXml(array(
			'xmlString' => $this->_createOrderXml(array(
				'order' => $this->_getOrderInfo(trim(strip_tags($key)))
			))
		));
		
		if(isset($response->status) && $response->status == 0)
		{
			switch($response->error->code)
			{
				case 420: 
					JError::raiseWarning('WARNING', JText::_('Uw logingegevens van Acumulus zijn onjuist.'));
					break;
				default: 
					JError::raiseWarning('WARNING', JText::_('Fout tijdens verbinding met de Acumulus server: ' . $response->error->message));
					break;
			}
		}
	}
	
	/**
	 * Post xml
	 * 
	 * @param array $params
	 * @return string
	 */
	private function _postXml($params)
	{
		if(!isset($params['xmlString']) || empty($params['xmlString']))
		{
			return '';
		}
		$xmlString = urlencode($params['xmlString']);
		
		$ch = curl_init();
		curl_setopt($ch, CURLOPT_URL, 'https://www.sielsystems.nl/acumulus/api/add_invoice.php');
		curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);
		curl_setopt($ch, CURLOPT_POST, 1);
		curl_setopt($ch, CURLOPT_POSTFIELDS, "xmlstring=$xmlString");
		curl_setopt($ch, CURLOPT_TIMEOUT, 10);
		$response = curl_exec($ch);
		
		if(curl_errno($ch))
		{
			switch(curl_errno($ch))
			{
				case 60:
					curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, FALSE);
					curl_setopt($ch, CURLOPT_SSL_VERIFYHOST, FALSE);
					$response = curl_exec($ch);
					break;
			}
		}
		
		curl_close($ch);
		
		return simplexml_load_string($response);
	}
	
	/** 
	 * Create order xml
	 *
	 * @param array $params
	 * @return string
	 */
	private function _createOrderXml($params)
	{
		if(!isset($params['order']) || !is_object($params['order']))
		{
			return '';
		}
		$order = $params['order'];
		$user = $this->_getUserInfo($order->virtuemart_user_id, $order->virtuemart_order_id);
		$moduleConfig = $this->_getModuleConfig();
		if($moduleConfig->get('acumulus_bankaccountnumber') != ''){
			$accountnumber = trim(strip_tags($moduleConfig->get('acumulus_bankaccountnumber')));
		}else{
			$accountnumber = '';
		}
		if($moduleConfig->get('acumulus_costheading') != ''){
			$costheading = trim(strip_tags($moduleConfig->get('acumulus_costheading')));
		}else{
			$costheading = '';
		}
		if($moduleConfig->get('acumulus_template') != ''){
			$template = trim(strip_tags($moduleConfig->get('acumulus_template')));
		}else{
			$template = '';
		}

		/* AD 2013-6-17 bepaling of order direct betaald is of niet, waarmee */
		/* status direct op betaald of niet wordt gezet. Wordt gedaan adhv   */
		/* virtuemart_paymentmethod_id. Type 1 en 6 zijn gecheckte online    */
		/* methodes die direct op betaald mogen als Confirmed. Alle andere   */
		/* niet, moeten op niet betaald en met de hand qua status en betaal  */
		/* datum worden gezet in Acumulus.                                   */
		$betaald = '0';
		if ($order->virtuemart_paymentmethod_id==1 || $order->virtuemart_paymentmethod_id==6)
		{
			$betaald = '1';
		}

		/* AD 2013-6-18 bij description order_id vervangen door order_number */
		/* klanten gebruiken het volledige nummer bij betalingen.			 */
		/* Grote en kleine letter gebruik bij fullname aangepast en andere   */
		/* door klanten rechtstreeks ingevoerde data geuniformeerd.          */
		/* Klantnaam verbeterd door spatie teveel bij geen middle naam weg.  */
		/* Betaald afhankelijk gemaakt van betaal methode.                   */
		

		$postcode = substr(ltrim($user["zip"]),0,4)." ".strtoupper(trim(substr(ltrim($user["zip"]),4,strlen($user["zip"])-4)));
		$klantnaam = ucfirst($user["first_name"]) . ' ';
		if ($user["middle_name"] != '')
		{
			$klantnaam .= strtolower($user["middle_name"]) . ' ';
		}
		$klantnaam .= ucfirst($user["last_name"]);
		
		return '<?xml version="1.0" encoding="UTF-8"?>
			<import>
				<contract>
					<code><![CDATA[' . $moduleConfig->get('acumulus_code') . ']]></code>
					<username><![CDATA[' . $moduleConfig->get('acumulus_username') . ']]></username>
					<password><![CDATA[' . $moduleConfig->get('acumulus_password') . ']]></password>
					<email_onerror><![CDATA[' . $moduleConfig->get('acumulus_error_email') . ']]></email_onerror>
					<customer>
						<type><![CDATA[' . $moduleConfig->get('acumulus_customer_type') . ']]></type>
						<companyname1><![CDATA[' . htmlspecialchars($user["company"]) . ']]></companyname1>
						<companyname2><![CDATA[]]></companyname2>
						<fullname><![CDATA[' . htmlspecialchars($klantnaam) . ']]></fullname>
						<salutation><![CDATA[]]></salutation>
						<address1><![CDATA[' . htmlspecialchars(ucfirst($user["address_1"])) . ']]></address1>
						<address2><![CDATA[' . htmlspecialchars(ucfirst($user["address_2"])) . ']]></address2>
						<postalcode><![CDATA[' . htmlspecialchars($postcode) . ']]></postalcode>
						<city><![CDATA[' . htmlspecialchars(ucfirst($user["city"])) . ']]></city>
						<locationcode><![CDATA[1]]></locationcode>
						<countrycode><![CDATA[' . htmlspecialchars($user["country"]) . ']]></countrycode>
						<vatnumber><![CDATA[]]></vatnumber>
						<telephone><![CDATA[' . htmlspecialchars($user["phone_1"]) . ']]></telephone>
						<fax><![CDATA[]]></fax>
						<email><![CDATA[' . htmlspecialchars(strtolower($user["email"])) . ']]></email>
						<overwriteifexists><![CDATA[0]]></overwriteifexists>
						<mark><![CDATA[]]></mark>
						<invoice>
							'.($moduleConfig->get('acumulus_number_type') == '2' ? '<number><![CDATA['.$order->order_number.']]></number>'."\n" : '').
							((int)$moduleConfig->get('acumulus_invoice_date') == 1 ? '<issuedate><![CDATA['.JFactory::getDate($order->created_on)->toFormat('%Y-%m-%d').']]></issuedate>'."\n" : '<issuedate><![CDATA[]]></issuedate>'."\n").
							'<concept><![CDATA[0]]></concept>
							<vattype><![CDATA[1]]></vattype>
                            <costheading><![CDATA[' . htmlspecialchars($costheading) . ']]></costheading>
                            <accountnumber><![CDATA[' . $accountnumber . ']]></accountnumber>
							<paymentstatus><![CDATA[' . $betaald . ']]></paymentstatus>
							<paymentdate><![CDATA[]]></paymentdate>
							<description><![CDATA[Ordernummer ' . $order->order_number . ']]></description>
                            <template><![CDATA[' . htmlspecialchars($template) . ']]></template>
							' . $this->_createOrderRowsXml(array('order' => $order)) . '	
							' . $this->_createShippingRowsXml(array('order' => $order)) . '
							' . $this->_createPaymentRowsXml(array('order' => $order)) . '
						</invoice>
					</customer>
				</contract>
			</import>';

	/* AD 2013-6-16 Tijdelijk er tussenuit gehaald. Korting komt bij osn (nog) zelden voor en   */
	/* de manier waarop deze in de koppeling wordt verwerkt (alleen bij 1 BTW percentage		*/
	/* is iets te grof.																			*/
	/*							' . $this->_createDiscountRowsXml(array('order' => $order)) . '	*/

	}
	
		
	/**
	 * Create order rows xml
	 *
	 * @param array @params
	 * @return string
	 */
	private function _createOrderRowsXml($params)
	{//print_r($params); die();

		if(!isset($params['order']) || !is_object($params['order']))
		{
			return '';
		}
		
		$order = $params['order'];
		$rows = '';
		foreach($this->_getOrderItems($order->virtuemart_order_id) as $orderRow)
		{
			$this->p_tax = round($orderRow->product_tax / $orderRow->product_item_price * 100);
			$rows .= '
				<line>
					<product><![CDATA[' . htmlspecialchars($orderRow->order_item_name) . ']]></product>
					<unitprice><![CDATA[' . number_format($orderRow->product_item_price, 4, '.', '') . ']]></unitprice>
					<vatrate><![CDATA['.round($orderRow->product_tax / $orderRow->product_item_price * 100).']]></vatrate>
					<quantity><![CDATA[' . $orderRow->product_quantity . ']]></quantity>
					<costprice><![CDATA[]]></costprice>
				</line>';
		}
		
		return $rows;
	}
	
	/**
	 * Create discount rows xml
	 *
	 * @param array @params
	 * @return string
	 */
	private function _createDiscountRowsXml($params)
	{
		if(!isset($params['order']) || !is_object($params['order']))
		{
			return '';
		}
		
		$order = $params['order'];
		
		$rows = '';
		if($order->order_billDiscountAmount != 0)
		{
			/*$rows .= '
				<line>
					<product><![CDATA[Toeslag]]></product>
					<unitprice><![CDATA[-' . str_replace('-', '', number_format($order->order_billDiscountAmount / (100 + $this->p_tax) * 100), 2, '.', '')) . ']]></unitprice>
					<vatrate><![CDATA[' . $this->p_tax. ']]></vatrate>
					<quantity><![CDATA[1]]></quantity>
					<costprice><![CDATA[]]></costprice>
				</line>';*/
			$rows .= '
				<line>
					<product><![CDATA[Toeslag]]></product>
					<unitprice><![CDATA[-' .  str_replace('-', '', number_format(($order->order_billDiscountAmount / (100 + $this->p_tax) * 100), 2, '.', '')) . ']]></unitprice>
					<vatrate><![CDATA[' . $this->p_tax. ']]></vatrate>
					<quantity><![CDATA[1]]></quantity>
					<costprice><![CDATA[]]></costprice>
				</line>';
		}
		
		if($order->coupon_discount != 0)
		{
			$rows .= '
				<line>
					<product><![CDATA[Coupon]]></product>
					<unitprice><![CDATA[-' . str_replace('-', '', number_format(($order->coupon_discount / (100 + $this->p_tax) * 100), 2, '.', '')) . ']]></unitprice>
					<vatrate><![CDATA[' . $this->p_tax. ']]></vatrate>
					<quantity><![CDATA[1]]></quantity>
					<costprice><![CDATA[]]></costprice>
				</line>';
		}
		
		return $rows;
	}	
	
	/**
	 * Create shipping rows xml
	 *
	 * @param array @params
	 * @return string
	 */
	private function _createShippingRowsXml($params)
	{
		if(!isset($params['order']) || !is_object($params['order']))
		{
			return '';
		}
		
		$order = $params['order'];

		$rows = '';
		if($order->order_shipment > 0)
		{
			/*$taxDetails = array_keys(unserialize($order->product_tax));
			$shippingTax = (isset($taxDetails[0])) ? $taxDetails[0] : $order->order_shipment_tax;

			$rows .= '
				<line>
					<product><![CDATA[Verzending]]></product>
					<unitprice><![CDATA[' . number_format($order->order_shipment, 2, '.', '') . ']]></unitprice>
					<vatrate><![CDATA[' . $shippingTax * 100 . ']]></vatrate>
					<quantity><![CDATA[1]]></quantity>
					<costprice><![CDATA[]]></costprice>
				</line>';*/
				/* AD 2013-6-16 aangepast zodat de werkelijk gehanteerd verzendkosten en gehanteerde */
				/* BTW percentage uit de winkel worden gehanteerd ipv iets berekends in de koppeling */
				$rows .= '
				<line>
					<product><![CDATA[Verzending]]></product>
					<unitprice><![CDATA[' . number_format($order->order_shipment, 4, '.', '') . ']]></unitprice>
					<vatrate><![CDATA[' .round($order->order_shipment_tax / $order->order_shipment * 100). ']]></vatrate>
					<quantity><![CDATA[1]]></quantity>
					<costprice><![CDATA[]]></costprice>
				</line>';
		}
		
		return $rows;
	}

	/**
	 * Create payment costs rows xml
	 * AD 2013-6-18
	 * @param array @params
	 * @return string
	 */
	private function _createPaymentRowsXml($params)
	{
		if(!isset($params['order']) || !is_object($params['order']))
		{
			return '';
		}
		
		$order = $params['order'];

		$rows = '';
		if($order->order_payment > 0)
		{
			/* AD 2013-6-18 De werkelijk gehanteerde kosten en gehanteerd */
			/* BTW percentage van de kosten voor de betalingsmethode.     */
			$rows .= '
				<line>
					<product><![CDATA[Betaling]]></product>
					<unitprice><![CDATA[' . number_format($order->order_payment, 4, '.', '') . ']]></unitprice>
					<vatrate><![CDATA[' .round($order->order_payment_tax / $order->order_payment * 100). ']]></vatrate>
					<quantity><![CDATA[1]]></quantity>
					<costprice><![CDATA[]]></costprice>
				</line>';
		}
		
		return $rows;
	}
	
	/**
	 * Get order info
	 *
	 * @param int $orderId
	 * @return ps_DB
	 */
	private function _getOrderInfo($orderId)
	{
		$db = JFactory::getDBO();
		$db->setQuery("SELECT * FROM #__virtuemart_orders WHERE virtuemart_order_id = '".$db->getEscaped($orderId)."'");
		$result = $db->loadObjectList();
		return $result[0];
	}
	
	/**
	 * Get order items
	 *
	 * @param int $orderId
	 * @return array
	 */
	private function _getOrderItems($orderId)
	{
		$db = JFactory::getDBO();
		$query = "SELECT * FROM #__virtuemart_order_items WHERE virtuemart_order_id = '".$orderId."' AND virtuemart_product_id = #__virtuemart_order_items.virtuemart_product_id";
		$db->setQuery($query);
		return $db->loadObjectList();
	}
	
	/**
	 * Get user info
	 *
	 * @param int $userId
	 * @return ps_DB
	 */
	private function _getUserInfo($userId, $orderId)
	{	
		$db = JFactory::getDBO();
		$query = "SELECT * FROM #__virtuemart_order_userinfos WHERE virtuemart_order_id = '".$orderId."' AND virtuemart_user_id = '".$userId."'";
		$db->setQuery($query);
		$row = $db->loadAssoc();

		return $row;
	}
	
	/**
	 * Get module config
	 *
	 * @return JParameter
	 */
	private function _getModuleConfig()
	{
		$db = JFactory::getDBO();
		$db->setQuery('SELECT params FROM #__modules WHERE module = "mod_virtuemart_acumulus"');
        $params = $db->loadObjectList();
		$params = new JParameter($params[0]->params);
		
		return $params;
	}
}

?>